Documentation
¶
Overview ¶
Package list provides the functionality for ajfs list command.
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
Types ¶
type Config ¶
type Config struct {
config.CommonConfig
DisplayFullPaths bool // If true then each path entry will be prefixed with the root path of the database.
DisplayHashes bool // Display file signature hashes if available.
DisplayMinimal bool // Display only the paths.
}
Config for the ajfs list command.
Click to show internal directories.
Click to hide internal directories.